<?php

namespace App\Service;

use App\Entity\Client;
use App\Exception\ClientAlreadyExistsException;
use App\Exception\ClientNotExistsException;
use App\Repository\ClientRepository;
use Psr\Clock\ClockInterface;
use Symfony\Component\Uid\Uuid;

class ClientService
{
    public function __construct(
        public readonly ClientRepository $clientRepository,
        public readonly ClockInterface $clock,
    ) {
    }

    /**
     * @throws \App\Exception\ClientNotExistsException
     */
    public function getClient(Uuid $clientId): Client
    {
        $client = $this->clientRepository->find($clientId);

        if (!$client) {
            throw new ClientNotExistsException(
                message: sprintf('Client with id "%s" does not exist.', $clientId->toRfc4122())
            );
        }

        return $client;
    }

    public function createNewClient(Client $client): Client
    {
        $existing = $this->clientRepository->find($client->getClientId());

        if ($existing !== null) {
            throw new ClientAlreadyExistsException(
                message: sprintf('Client with id "%s" already exists.', $client->getClientId()->toRfc4122())
            );
        }

        return $this->clientRepository->save($client);
    }

    public function deduceFromBalance(Client $client, int $amount): void
    {
        $client->deduceFromBalance($amount);

        $this->clientRepository->save($client);
    }

    public function topUpBalance(Client $client, int $amount): void
    {
        $client->topUpBalance($amount);

        $this->clientRepository->save($client);
    }

    public function lock(Client $client): void
    {
        $client->setLockedAt($this->clock->now());

        $this->clientRepository->save($client);
    }

    public function unlock(Client $client): void
    {
        $client->setLockedAt(null);

        $this->clientRepository->save($client);
    }
}
